#!/usr/bin/python

import socketserver
import struct
import sys

TIMEOUT = 1
DEBUG = False

class MyHandler(socketserver.StreamRequestHandler):
    def handle(self):
        qtype = 99
        answer = 1
        query = "Invalid"
        try:
            data = self.request.recv(1)
            if DEBUG:
                print("Command read", file=sys.stderr)
            qtype = struct.unpack("B", data)[0]
            answer = 0
            match qtype:
              case 1:
                data = self.request.recv(1) 
                if DEBUG:
                    print("Data read for command 1", file=sys.stderr)
                size = struct.unpack("B", data)[0]
                data = self.request.recv(size)
                if len(data) != size:
                    answer = 1
                    query = "Wrong length"
                else:
                    query = data.decode()
              case 2:
                data = self.request.recv(4) 
                if DEBUG:
                    print("Data read for command 2", file=sys.stderr)
                query = struct.unpack("I", data)[0]
              case 3:
                query = None
              case _:
                answer = 1
                query = "Invalid"
        except struct.error:
            answer = 1
            query = "Invalid"
        self.request.setblocking(0)
        # Check if there is extraneous data
        if DEBUG:
            print("Checking extra data", file=sys.stderr)
        try:
            data = self.request.recv(1)
            answer = 1
            query = "Extra data"            
        except BlockingIOError:
            pass
        print("Received \"{}/{}\" from {}, I return {}".format(qtype, query, self.client_address[0], answer))    
        self.request.sendall(struct.pack("B", answer))

class MyServer(socketserver.ThreadingMixIn, socketserver.TCPServer):
    def __init__(self):
        self.allow_reuse_address = True
        super().__init__(("", 9999), MyHandler)
    
server = MyServer()
server.serve_forever()

